"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateContentModelTypescriptCode = exports.parseContentModelSchema = void 0;
var ss = __importStar(require("superstruct"));
var ContentModelStringKind = ss.enums([
    "string",
    "textarea",
    "image_file_id",
    "file_id",
    "product_id",
    "variant_id",
    "category_id",
    "brand_id",
    "content_block_id",
    "web_url",
    "dropdown",
    "radio",
]);
var ContentModelNumberKind = ss.enums(["number", "percentage", "rating"]);
var ContentModelBooleanKind = ss.enums(["checkbox", "toggle"]);
var ContentModelDatetimeKind = ss.enums(["date", "datetime"]);
var ContentModelEnumValue = ss.type({
    label: ss.string(),
    value: ss.string(),
});
var ContentModelStringValidation = ss.type({
    minLength: ss.optional(ss.number()),
    minLengthError: ss.optional(ss.string()),
    maxLength: ss.optional(ss.number()),
    maxLengthError: ss.optional(ss.string()),
    pattern: ss.optional(ss.string()),
    patternError: ss.optional(ss.string()),
    enum: ss.optional(ss.array(ContentModelEnumValue)),
    defaultValue: ss.optional(ss.string()),
});
var ContentModelNumberValidation = ss.type({
    min: ss.optional(ss.number()),
    minError: ss.optional(ss.string()),
    max: ss.optional(ss.number()),
    maxError: ss.optional(ss.string()),
    integer: ss.optional(ss.boolean()),
});
var ContentModelTypeFieldProps = ss.type({
    name: ss.string(),
    label: ss.optional(ss.string()),
});
var ContentModelType_String = ss.type({
    type: ss.literal("string"),
    kind: ContentModelStringKind,
    validation: ss.optional(ContentModelStringValidation),
    help: ss.optional(ss.string()),
});
var ContentModelType_Datetime = ss.type({
    type: ss.literal("datetime"),
    kind: ContentModelDatetimeKind,
    help: ss.optional(ss.string()),
});
var ContentModelType_Boolean = ss.type({
    type: ss.literal("boolean"),
    kind: ContentModelBooleanKind,
    help: ss.optional(ss.string()),
});
var ContentModelType_Number = ss.type({
    type: ss.literal("number"),
    kind: ContentModelNumberKind,
    validation: ContentModelNumberValidation,
    help: ss.optional(ss.string()),
});
var NonObjectContentModelType = ss.union([
    ContentModelType_String,
    ContentModelType_Datetime,
    ContentModelType_Boolean,
    ContentModelType_Number,
]);
var ContentModelObjectField = ss.intersection([NonObjectContentModelType, ContentModelTypeFieldProps]);
var ContentModelType_Object = ss.type({
    type: ss.literal("object"),
    fields: ss.array(ContentModelObjectField),
    help: ss.optional(ss.string()),
});
var ContentModelType = ss.union([ContentModelType_Object, NonObjectContentModelType]);
var ContentModelSchema = ss.type({
    label: ss.optional(ss.string()),
    name: ss.string(),
    json: ContentModelType,
});
function parseContentModelSchema(data) {
    var j;
    try {
        j = JSON.parse(data);
    }
    catch (err) {
        throw new Error("failed parsing content model schema: invalid json");
    }
    var r = ss.validate(j, ContentModelSchema);
    if (!r[0]) {
        return r[1];
    }
    else {
        var additionalInfo = r[0]
            .failures()
            .map(function (f) { return "  ".concat(f.message); })
            .join("\n");
        throw new Error("failed parsing content model schema: ".concat(r[0].message, "\ndetailed report:\n").concat(additionalInfo));
    }
}
exports.parseContentModelSchema = parseContentModelSchema;
function generateTypeFor(v) {
    var _a;
    switch (v.type) {
        case "boolean":
            return "ss.boolean()";
        case "string":
            var e = (_a = v.validation) === null || _a === void 0 ? void 0 : _a.enum;
            if (e) {
                return "ss.enums([".concat(e.map(function (v) { return "".concat(JSON.stringify(v.value), " as const"); }).join(","), "])");
            }
        case "datetime":
            return "ss.string()";
        case "number":
            return "ss.number()";
        case "object":
            var fields = [];
            for (var _i = 0, _b = v.fields; _i < _b.length; _i++) {
                var f = _b[_i];
                fields.push("".concat(JSON.stringify(f.name), ": ").concat(generateTypeFor(f)));
            }
            return "ss.type({".concat(fields.join(","), "})");
    }
}
function generateContentModelTypescriptCode(schema) {
    var out = "";
    out += "import * as ss from \"superstruct\";\n";
    out += "export const ContentModel = ".concat(generateTypeFor(schema.json), ";\n");
    return out;
}
exports.generateContentModelTypescriptCode = generateContentModelTypescriptCode;
